package cn.ecpark.service.usergw.biz.filters;

import java.nio.ByteBuffer;

import com.alibaba.fastjson.JSON;

import org.apache.dubbo.config.ApplicationConfig;
import org.apache.dubbo.config.ReferenceConfig;
import org.apache.dubbo.config.utils.ReferenceConfigCache;
import org.apache.dubbo.rpc.service.GenericService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.cloud.gateway.filter.GatewayFilterChain;
import org.springframework.cloud.gateway.filter.GlobalFilter;
import org.springframework.context.ApplicationContext;
import org.springframework.core.Ordered;
import org.springframework.http.server.reactive.ServerHttpResponse;
import org.springframework.stereotype.Component;
import org.springframework.web.server.ServerWebExchange;

import cn.ecpark.service.usergw.biz.filters.bean.GenericServicePool;
import io.netty.util.Recycler;
import lombok.extern.slf4j.Slf4j;
import reactor.core.publisher.Mono;

@Component
@Slf4j
public class DubboFilter implements GlobalFilter, Ordered {

    @Autowired
    private ApplicationContext appContext;

    @Override
    public int getOrder() {
        return Ordered.LOWEST_PRECEDENCE;
    }

    @Override
    public Mono<Void> filter(ServerWebExchange exchange, GatewayFilterChain chain) {
        // exchange.getRequest();
    
        String application = "dubbo-exchange"; // 必须
        // reference.setRegistry(new RegistryConfig("zookeeper://127.0.0.1:2181"));
        String rgroup = "group";
        String rinterface = "ocean.demo.api.IExchange"; // 弱类型接口名 必须
        String rversion  = "1.0.0";
        String rmethod = "Say";

        
        String rkey = "";
        rkey += application;

        if(rgroup != ""){
            rkey += "/" + rgroup;
        }

        rkey += "/" + rinterface;      
        rkey += ":" + rversion;
        
        GenericServicePool gsPool = appContext.getBean(GenericServicePool.class);
        if(!gsPool.contains(rkey)) {

            ReferenceConfig<GenericService> reference = new ReferenceConfig<GenericService>(); // 该实例很重量，里面封装了所有与注册中心及服务提供方连接，请缓存
        
            reference.setApplication(new ApplicationConfig(application)); // 必须
            // reference.setRegistry(new RegistryConfig("zookeeper://127.0.0.1:2181"));
            // reference.setGroup("group");
            reference.setInterface(rinterface); // 弱类型接口名 必须
            reference.setVersion(rversion); // 必须
            reference.setGeneric(true); // 声明为泛化接口

            gsPool.add(rkey, reference.get());
        }

        GenericService gs = gsPool.get(rkey);

        Object result = gs.$invoke(rmethod, new String[] {"java.lang.String"}, new Object[] {"213"});
        
        if (result != null) {
            ServerHttpResponse response = exchange.getResponse();
            return response.writeWith(
                    Mono.just(response.bufferFactory().wrap(ByteBuffer.wrap(JSON.toJSONString(result).getBytes()))));
        }
        return chain.filter(exchange);
    }

}