package cn.ecpark.service.usergw.biz.filters.bean;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.concurrent.TimeUnit;
import java.util.concurrent.locks.Condition;
import java.util.concurrent.locks.Lock;
import java.util.concurrent.locks.ReentrantLock;

import org.apache.dubbo.config.ReferenceConfig;
import org.apache.dubbo.rpc.service.GenericService;

import lombok.extern.slf4j.Slf4j;

/**
 * GenericServicePool
 */
@Slf4j
public class GenericServicePool {

    private class GenericServiceManager {
        Lock idleLock = new ReentrantLock();
        LinkedList<GenericService> idle = new LinkedList<>();

        private String key;

        @Override
        public int hashCode() {
            return this.key.hashCode();
        }

        /**
         * @return String return the key
         */
        public String getKey() {
            return key;
        }

        /**
         * @param key the key to set
         */
        public void setKey(String key) {
            this.key = key;
        }

        public GenericService get() {
            try {
                if (idleLock.tryLock(15L, TimeUnit.SECONDS)) {
                    while (idle.isEmpty()) {
                        idle.wait();
                    }
                    return idle.pop();
                } else {
                    log.error(this.key + ": 超时{}秒", 15);
                }
            } catch (Exception e) {
                // TODO: handle exception
                log.error("悠闲队列出现问题: " + e.toString());
            } finally {
                idleLock.unlock();
            }
            return null;
        }

        public void add(GenericService gs) {
            idle.add(gs);
            if (idle.isEmpty()) {
                idle.notify();
            }
        }



        public GenericServiceManager() {
        }

    }

    HashMap<String, GenericServiceManager> gsDictionary;

    public GenericServicePool() {
        gsDictionary = new HashMap<String, GenericServiceManager>();
    }

    public GenericService get(String key) {
        GenericServiceManager pool = gsDictionary.get(key);
        return pool.get();
    }

    public void add(String key, GenericService genericService) {
        GenericServiceManager pool;
        if (gsDictionary.containsKey(key)) {
            pool = gsDictionary.get(key);
        } else {
            pool = new GenericServiceManager();
            gsDictionary.put(key, pool);
        }
        pool.add(genericService);
    }

    public boolean contains(String key) {
        return gsDictionary.containsKey(key);
    }
}