package cn.ecpark.service.usergw.biz.filters.factory;

import java.nio.ByteBuffer;
import java.util.ArrayList;
import java.util.List;

import com.alibaba.fastjson.JSON;

import org.apache.dubbo.config.ApplicationConfig;
import org.apache.dubbo.config.ReferenceConfig;
import org.apache.dubbo.rpc.service.GenericService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.cloud.gateway.filter.GatewayFilter;
import org.springframework.cloud.gateway.filter.factory.AbstractGatewayFilterFactory;
import org.springframework.context.ApplicationContext;
import org.springframework.http.server.reactive.ServerHttpResponse;
import org.springframework.stereotype.Component;

import cn.ecpark.service.usergw.biz.filters.bean.GenericServicePool;
import reactor.core.publisher.Mono;;

@Component
public class DubboFilterFactory extends AbstractGatewayFilterFactory<DubboFilterFactory.Config> {

    @Autowired
    private ApplicationContext appContext;

    public DubboFilterFactory() {
        super(Config.class);
    }

    @Override
    public GatewayFilter apply(Config config) {
        String dubboUri = config.dubboUri;

        return (exchange, chain) -> {

            String application = "dubbo-exchange"; // 必须
            // reference.setRegistry(new RegistryConfig("zookeeper://127.0.0.1:2181"));
            String rgroup = "group";
            String rinterface = "ocean.demo.api.IExchange"; // 弱类型接口名 必须
            String rversion  = "1.0.0";
            String rmethod = "Say";
            List<String> rparamTypes = new ArrayList<String>();
            List<String> rparams = new ArrayList<String>();
    
            
            String rkey = "";
            rkey += application;
    
            if(rgroup != ""){
                rkey += "/" + rgroup;
            }
    
            rkey += "/" + rinterface;      
            rkey += ":" + rversion;
            
            GenericServicePool gsPool = appContext.getBean(GenericServicePool.class);
            if(!gsPool.contains(rkey)) {
    
                ReferenceConfig<GenericService> reference = new ReferenceConfig<GenericService>(); // 该实例很重量，里面封装了所有与注册中心及服务提供方连接，请缓存
            
                reference.setApplication(new ApplicationConfig("dubbo-exchange")); // 必须
                // reference.setRegistry(new RegistryConfig("zookeeper://127.0.0.1:2181"));
                reference.setGroup("group");
                reference.setInterface("ocean.demo.api.IExchange"); // 弱类型接口名 必须
                reference.setVersion("1.0.0"); // 必须
                reference.setGeneric(true); // 声明为泛化接口
    
                gsPool.add(rkey, reference.get());
            }
    
            GenericService gs = gsPool.get(rkey);
            Object result = gs.$invoke("Hello", new String[] {}, new Object[] {});
            
            if (result != null) {
                ServerHttpResponse response = exchange.getResponse();
                return response.writeWith(
                        Mono.just(response.bufferFactory().wrap(ByteBuffer.wrap(JSON.toJSONString(result).getBytes()))));
            }
            return chain.filter(exchange);
            
            // gs.$invoke(method, parameterTypes, args)
        };
    }

    
    public static class Config {

		private String dubboUri;

		public String getDubboUri() {
			return dubboUri;
		}

		public void setDubboUri(String dubboUri) {
			this.dubboUri = dubboUri;
		}

	}


    
}